import type { Project, ResolvedModule } from '@callstack/polygen-project';
import { CodegenContext } from './codegen/context.js';
import type { W2CExternModule, W2CGeneratedModule } from './codegen/modules.js';
import { OutputGenerator } from './helpers/output-generator.js';
import type { Plugin } from './plugin.js';
export { FileExternallyChangedError, FileOverwriteError, } from './helpers/output-generator.js';
/**
 * Options used to change W2CGenerator behavior.
 */
export interface CodegenOptions {
    /**
     * Path to output directory where generated files will be created.
     *
     * Directories for specific modules will be created within this
     * directory.
     */
    outputDirectory?: string;
    /**
     * If true, all modules are built into single turbo module.
     *
     * This can increase build times if one of the WASM module was changed.
     * This only works with static linking.
     *
     * TODO: Currently only this works
     *
     * @defaultValue true
     */
    singleProject?: boolean;
    /**
     * If true, generated files are re-generated and written
     * even if stale.
     *
     * @defaultValue false
     */
    forceGenerate?: boolean;
    /**
     * If true, additional debug metadata files are generated alongside
     * output source files.
     *
     * @defaultValue false
     */
    generateMetadata?: boolean;
}
export interface PluginDispatchOptions {
    beforePluginDispatch?: (plugin: Plugin) => Promise<unknown> | unknown;
    afterPluginDispatch?: (plugin: Plugin) => Promise<unknown> | unknown;
}
/**
 * Main generator class for WebAssembly to C code generation.
 *
 * This class is responsible for generating C code from WebAssembly modules, and
 * contains all state and configuration necessary for the generation process.
 *
 * The generator is not reentrant, and should be used only once.
 */
export declare class Codegen {
    /**
     * The project configuration to generate code for.
     */
    readonly project: Project;
    /**
     * Configuration options for the generation process.
     */
    readonly options: CodegenOptions;
    /**
     * Shared context for all modules.
     *
     * Holds codegen state and module information.
     */
    readonly context: CodegenContext;
    /**
     * Collection of plugins to dispatch.
     */
    plugins: Plugin[];
    /**
     * Output generator instance used to write generated files to disk.
     */
    private readonly generator;
    /**
     * Creates a new W2CGenerator instance for the specified project and options.
     *
     * @see create
     */
    private constructor();
    /**
     * Creates a new W2CGenerator instance for the specified project and options.
     *
     * @param project The project configuration to generate code for.
     * @param options Optional configuration for the generation process.
     */
    static create(project: Project, options?: CodegenOptions): Promise<Codegen>;
    /**
     * Path to output directory where generated files will be created.
     */
    get outputDirectory(): string;
    /**
     * Generates all code for specified WebAssembly module.
     *
     * This includes generating the module exports bridge and the JavaScript
     *
     * @param module The module configuration to generate code for.
     */
    generateModule(module: ResolvedModule, options?: PluginDispatchOptions): Promise<[W2CGeneratedModule, W2CExternModule[]]>;
    /**
     * Generates the imported module by creating necessary files and configurations in the specified output directory.
     *
     * @param module The imported module data that needs to be processed and generated.
     * @return A promise that resolves when the module generation process is completed.
     */
    generateImportedModule(module: W2CExternModule): Promise<void>;
    /**
     * Returns the output generator for the host module.
     */
    get hostModuleOutput(): OutputGenerator;
    /**
     * Generates the host module and its corresponding bridges using the specified context and options.
     *
     * @param context The shared context containing module information and configurations.
     * @return A promise that resolves with the results of generating bridges for imported modules. Each promise result contains the status of the operation (fulfilled or rejected).
     */
    generateHostModule(): Promise<void>;
    /**
     * Finalizes the generation process by writing the generated files to the output directory.
     */
    finalize(): Promise<void>;
    get rootOutput(): OutputGenerator;
    private outputPathForModule;
}
//# sourceMappingURL=codegen.d.ts.map