import fs from 'node:fs/promises';
import path from 'node:path';
import { fileURLToPath } from 'node:url';
import { DEFAULT_PLUGINS } from './codegen-pipeline.js';
import { CodegenContext } from './codegen/context.js';
import { generateModuleExportsBridge } from './generators/module-bridge.js';
import { OutputGenerator, } from './helpers/output-generator.js';
import * as templates from './templates/library/index.js';
export { FileExternallyChangedError, FileOverwriteError, } from './helpers/output-generator.js';
const UMBRELLA_PROJECT_NAME = 'host';
const ASSETS_DIR = path.join(path.dirname(fileURLToPath(import.meta.url)), '../assets');
/**
 * Main generator class for WebAssembly to C code generation.
 *
 * This class is responsible for generating C code from WebAssembly modules, and
 * contains all state and configuration necessary for the generation process.
 *
 * The generator is not reentrant, and should be used only once.
 */
export class Codegen {
    /**
     * Creates a new W2CGenerator instance for the specified project and options.
     *
     * @see create
     */
    constructor(project, options = {}, previousWrittenFiles, plugins) {
        this.project = project;
        this.options = options;
        this.context = new CodegenContext();
        this.plugins = plugins ?? DEFAULT_PLUGINS;
        this.generator = new OutputGenerator({
            outputDirectory: this.outputDirectory,
            assetsDirectory: ASSETS_DIR,
            forceGenerate: options.forceGenerate,
        }, previousWrittenFiles);
    }
    /**
     * Creates a new W2CGenerator instance for the specified project and options.
     *
     * @param project The project configuration to generate code for.
     * @param options Optional configuration for the generation process.
     */
    static async create(project, options = {}) {
        let previouslyWrittenFiles;
        try {
            const outputDir = options.outputDirectory ?? project.paths.fullOutputDirectory;
            const previousMap = await fs.readFile(path.join(outputDir, 'polygen-output.json'), { encoding: 'utf-8' });
            previouslyWrittenFiles = JSON.parse(previousMap).files;
        }
        catch (e) {
            if (e && typeof e === 'object' && 'code' in e && e.code !== 'ENOENT') {
                throw e;
            }
        }
        return new Codegen(project, options, previouslyWrittenFiles);
    }
    /**
     * Path to output directory where generated files will be created.
     */
    get outputDirectory() {
        return (this.options.outputDirectory ?? this.project.paths.fullOutputDirectory);
    }
    /**
     * Generates all code for specified WebAssembly module.
     *
     * This includes generating the module exports bridge and the JavaScript
     *
     * @param module The module configuration to generate code for.
     */
    async generateModule(module, options = {}) {
        const [moduleContext, imports] = await this.context.addModule(module);
        const generator = this.generator.forPath(this.outputPathForModule(module, moduleContext.name));
        await generateModuleExportsBridge(generator, moduleContext, {
            forceGenerate: this.options.forceGenerate,
            outputConfig: this.project.options.output,
            moduleConfig: module,
        });
        const pluginContext = {
            codegen: this,
            moduleOutput: generator,
            context: moduleContext,
            module,
        };
        for (const plugin of this.plugins) {
            await options.beforePluginDispatch?.(plugin);
            await plugin.moduleGenerated?.(pluginContext);
            await options.afterPluginDispatch?.(plugin);
        }
        return [moduleContext, imports];
    }
    /**
     * Generates the imported module by creating necessary files and configurations in the specified output directory.
     *
     * @param module The imported module data that needs to be processed and generated.
     * @return A promise that resolves when the module generation process is completed.
     */
    async generateImportedModule(module) {
        const generator = this.hostModuleOutput;
        await generator.writeAllTo({
            [`${module.name}-imports.h`]: templates.buildImportBridgeHeader(module),
            [`${module.name}-imports.cpp`]: templates.buildImportBridgeSource(module),
        });
    }
    /**
     * Returns the output generator for the host module.
     */
    get hostModuleOutput() {
        return this.generator.forPath(UMBRELLA_PROJECT_NAME);
    }
    /**
     * Generates the host module and its corresponding bridges using the specified context and options.
     *
     * @param context The shared context containing module information and configurations.
     * @return A promise that resolves with the results of generating bridges for imported modules. Each promise result contains the status of the operation (fulfilled or rejected).
     */
    async generateHostModule() {
        const pluginContext = {
            codegen: this,
            rootOutput: this.generator,
            projectOutput: this.hostModuleOutput,
            generatedModules: this.context.modules,
        };
        for (const plugin of this.plugins) {
            await plugin.hostProjectGenerated?.(pluginContext);
        }
    }
    /**
     * Finalizes the generation process by writing the generated files to the output directory.
     */
    async finalize() {
        const generatedMapPath = this.generator.outputPathTo('polygen-output.json');
        const resolvedExternalModules = await this.project.modules.getExternalModules();
        const contents = {
            files: this.generator.writtenFiles,
            externalPackages: Object.fromEntries(resolvedExternalModules.map((m) => [m.packageName, m.resolvedPath])),
        };
        await fs.writeFile(generatedMapPath, JSON.stringify(contents, undefined, 2));
    }
    get rootOutput() {
        return this.generator;
    }
    outputPathForModule(module, name) {
        let dirName = module.kind === 'external' ? `${module.packageName}#${name}` : name;
        if (this.options.singleProject) {
            return path.join(UMBRELLA_PROJECT_NAME, dirName);
        }
        return dirName;
    }
}
