import path from 'node:path';
import { mangleModuleName } from '../wasm2c/mangle.js';
import { buildGeneratedSymbol } from './utils.js';
/**
 * Base class for all modules types.
 *
 * There are two types of modules in the codegen:
 * - `W2CExternModule` - represents an imported module
 * - `W2CGeneratedModule` - represents a generated module wrapping a WebAssembly module.
 */
export class W2CModuleBase {
    constructor(name) {
        this.name = name;
        this.mangledName = mangleModuleName(name);
    }
    /**
     * Gets the C typename that was generated by wasm2c for this imported module.
     */
    get generatedContextTypeName() {
        return `w2c_${this.mangledName}`;
    }
}
/**
 * Represents an imported module in the shared context.
 *
 * This represents an imported module with imported symbols across all generated modules.
 *
 * Modules of this type have no implementation in WASM file, but the code for those modules
 * must be generated to allow defining them from JS code.
 */
export class W2CExternModule extends W2CModuleBase {
    constructor(name) {
        super(name);
        this.exports = new Map();
    }
    /**
     * Gets the name of the field in the root context struct.
     *
     * Each generated module that imports this module contains a field that holds
     * the imported module instance.
     */
    get generatedRootContextFieldName() {
        return `import_${this.mangledName}Ctx`;
    }
    addSymbol(symbol) {
        this.exports.set(symbol.localName, symbol);
    }
}
/**
 * Represents a generated module wrapping a WebAssembly module.
 *
 * This class is created for each WebAssembly module that is processed by the `wasm2c` tool.
 */
export class W2CGeneratedModule extends W2CModuleBase {
    constructor(context, body, checksum, sourceModulePath, moduleSpec) {
        const name = moduleSpec.wasm2c?.moduleName ?? path.basename(sourceModulePath, '.wasm');
        super(name);
        this.body = body;
        this.sourceModulePath = sourceModulePath;
        this.generatedClassName = capitalize(mangleModuleName(name));
        this.checksum = checksum;
        this.moduleImports = processImportedModulesInfo(this.body, context);
        this.imports = resolveImports(context, this.body);
        this.exports = processExports(this, this.body);
    }
    /**
     * Retrieves a reversed array of all imported modules.
     *
     * @return An array containing the imported modules in reverse order.
     */
    get importedModules() {
        return Object.values(this.moduleImports).toSorted((a, b) => a.name.localeCompare(b.name));
    }
    /**
     * Retrieves an array of all exported memories.
     */
    get exportedMemories() {
        return this.exports.filter((i) => i.target.kind === 'memory');
    }
    /**
     * Retrieves an array of all exported functions.
     */
    get exportedFunctions() {
        return this.exports.filter((i) => i.target.kind === 'function');
    }
    /**
     * Retrieves an array of all exported tables.
     */
    get exportedTables() {
        return this.exports.filter((i) => i.target.kind === 'table');
    }
    /**
     * Name of the function that creates a new instance of the module.
     */
    get moduleFactoryFunctionName() {
        return `get${this.generatedClassName}Module`;
    }
    /**
     * Name of the class that represents the module context.
     */
    get contextClassName() {
        return `${this.generatedClassName}ModuleContext`;
    }
}
function processImportedModulesInfo(module, codegenContext) {
    const importedModuleNames = new Set(module.imports.values().map((i) => i.module));
    const importsInfo = importedModuleNames
        .values()
        .map((moduleName) => [
        moduleName,
        codegenContext.getImportedModule(moduleName),
    ]);
    return Object.fromEntries(importsInfo);
}
function resolveImports(codegenContext, moduleBody) {
    return moduleBody.imports.map((el) => codegenContext.resolveImport(el.module, el.name));
}
function processExports(module, moduleBody) {
    return moduleBody.exports.map((el) => buildGeneratedSymbol(module, el));
}
function capitalize(name) {
    return name.charAt(0).toUpperCase() + name.slice(1);
}
