import { mangleModuleName, mangleName } from '../mangle.js';
/**
 * The `W2CModuleCodegenContext` class is responsible for maintaining the state and metadata
 * related to the code generation process of a WebAssembly (WASM) module using the `wasm2c` tool.
 * This class stores information about module imports and exports, and provides utilities
 * for handling the generated module code.
 */
export class W2CModuleCodegenContext {
    constructor(name, module) {
        /**
         * Collection of imported modules and their mangled names.
         */
        this.moduleImports = {};
        this.mangledName = mangleModuleName(name);
        this.moduleImports = processImportedModulesInfo(module);
        this.imports = processImports(this, module);
        this.exports = processExports(this, module);
    }
    /**
     * Gets a C typename that was generated by wasm2c for this module.
     */
    get generatedContextTypeName() {
        return `w2c_${this.mangledName}`;
    }
    /**
     * Retrieves a reversed array of all imported modules.
     *
     * @return An array containing the imported modules in reverse order.
     */
    get importedModules() {
        return Object.values(this.moduleImports).toSorted((a, b) => a.name.localeCompare(b.name));
    }
    /**
     * Retrieves an array of all exported memories.
     */
    get exportedMemories() {
        return this.exports.filter((i) => i.target.kind === 'memory');
    }
    /**
     * Retrieves an array of all exported functions.
     */
    get exportedFunctions() {
        return this.exports.filter((i) => i.target.kind === 'function');
    }
    /**
     * Retrieves an array of all exported tables.
     */
    get exportedTables() {
        return this.exports.filter((i) => i.target.kind === 'table');
    }
}
/**
 * Represents a WebAssembly to C (W2C) imported module with details about its name and mangled name.
 *
 * This class provides functionalities to handle and retrieve the context type
 * and root context field names for the imported module based on its mangled name.
 *
 * @remarks The mangled name is derived from the original module name through a specific mangle transformation.
 */
export class W2CCodegenLocalImportedModule {
    constructor(name) {
        this.name = name;
        this.mangledName = mangleModuleName(name);
    }
    /**
     * Gets the C typename that was generated by wasm2c for this imported module.
     */
    get generatedContextTypeName() {
        return `w2c_${this.mangledName}`;
    }
    /**
     * Gets the name of field of root context struct, that was generated by wasm2c for this imported module.
     */
    get generatedRootContextFieldName() {
        return `import_${this.mangledName}Ctx`;
    }
}
function processImportedModulesInfo(module) {
    const importedModuleNames = new Set(module.imports.values().map((i) => i.module));
    const importsInfo = importedModuleNames
        .values()
        .map((name) => [
        name,
        new W2CCodegenLocalImportedModule(name),
    ]);
    return Object.fromEntries(importsInfo);
}
function processImports(context, module) {
    return module.imports.map((el) => {
        const importInfo = context.moduleImports[el.module];
        if (!importInfo) {
            throw new Error(`Assert error: could not get import info for ${el.module}`);
        }
        const generatedFunctionName = mangleFunction(el.name, importInfo.mangledName);
        switch (el.target.kind) {
            case 'function':
                return {
                    ...el,
                    ...buildGeneratedFunctionInfo(el.target),
                    mangledName: mangleName(el.name),
                    moduleInfo: importInfo,
                    generatedFunctionName,
                };
            default:
                return {
                    ...el,
                    mangledName: mangleName(el.name),
                    moduleInfo: importInfo,
                    generatedFunctionName,
                };
        }
    });
}
function processExports(context, module) {
    return module.exports.map((el) => {
        const mangledName = mangleName(el.name);
        const generatedFunctionName = mangleFunction(el.name, context.mangledName);
        switch (el.target.kind) {
            case 'function':
                return {
                    ...el,
                    ...buildGeneratedFunctionInfo(el.target),
                    generatedFunctionName,
                    mangledName,
                };
            default:
                return {
                    ...el,
                    mangledName,
                    generatedFunctionName,
                };
        }
    });
}
function mangleFunction(name, mangledModule) {
    return `w2c_${mangledModule}_${mangleName(name)}`;
}
function buildGeneratedFunctionInfo(func) {
    const returnType = matchW2CRType(func.resultTypes[0]);
    return {
        parameterTypeNames: func.parametersTypes.map(matchW2CRType),
        returnTypeName: returnType,
    };
}
// TODO: workaround a bug(?) that all types are returned as either none, u32 or f64
function matchW2CRType(t) {
    if (!t) {
        return 'void';
    }
    // TODO: figure out why wasm2c returns u32 for number types sometimes
    if (t.startsWith('i')) {
        return t.replace(/^i/, 'u');
    }
    return t;
}
