import type { GeneratedExport, GeneratedFunctionExport, GeneratedImport } from "./types.js";
import { Module, type ModuleMemory, ModuleTable } from "@callstack/wasm-parser";
/**
 * Base class for all modules types
 */
export declare class W2CModuleBase {
    /**
     * Original name of the imported module.
     *
     * If module name is not passed, it defaults to module file name without the extension.
     *
     * Unsafe to use as a symbol in source code, use `mangledName` instead.
     */
    readonly name: string;
    /**
     * Mangled name of the imported module.
     *
     * Safe to use in source code as a symbol name.
     */
    readonly mangledName: string;
    constructor(name: string);
    /**
     * Gets the C typename that was generated by wasm2c for this imported module.
     */
    get generatedContextTypeName(): string;
}
/**
 * Represents an imported module in the shared context.
 *
 * This represents an imported module with imported symbols across all generated modules.
 *
 * Modules of this type have no implementation in WASM file, but the code for those modules
 * must be generated to allow defining them from JS code.
 */
export declare class W2CExternModule extends W2CModuleBase {
    /**
     * All symbols required to be provided.
     *
     * Generated based on imports of this module from another modules.
     */
    readonly symbols: GeneratedImport[];
    constructor(name: string, symbols: GeneratedImport[]);
    /**
     * Gets the name of the field in the root context struct.
     *
     * Each generated module that imports this module contains a field that holds
     * the imported module instance.
     */
    get generatedRootContextFieldName(): string;
}
/**
 * Represents a generated module wrapping a WebAssembly module.
 *
 * This class is created for each WebAssembly module that is processed by the `wasm2c` tool.
 */
export declare class W2CGeneratedModule extends W2CModuleBase {
    /**
     * Path to the file that the metadata was loaded from.
     */
    readonly sourceModulePath: string;
    /**
     * SHA-256 checksum of module contents
     */
    readonly checksum: Buffer;
    /**
     * Parsed WebAssembly module instance.
     */
    readonly body: Module;
    /**
     * Collection of generated module imports made by `wasm2c` tool.
     */
    readonly imports: GeneratedImport[];
    /**
     * Collection of generated module exports made by `wasm2c` tool.
     */
    readonly exports: GeneratedExport[];
    constructor(moduleBuffer: ArrayBuffer, sourceModulePath: string);
    /**
     * Retrieves a reversed array of all imported modules.
     *
     * @return An array containing the imported modules in reverse order.
     */
    get importedModules(): unknown[];
    /**
     * Retrieves an array of all exported memories.
     */
    get exportedMemories(): GeneratedExport<ModuleMemory>[];
    /**
     * Retrieves an array of all exported functions.
     */
    get exportedFunctions(): GeneratedFunctionExport[];
    /**
     * Retrieves an array of all exported tables.
     */
    get exportedTables(): GeneratedExport<ModuleTable>[];
}
//# sourceMappingURL=modules.d.ts.map