import { mangleModuleName, mangleName } from "./wasm2c/mangle.js";
import { Module } from "@callstack/wasm-parser";
import path from "node:path";
import { computeChecksumBuffer } from "./helpers/checksum.js";
/**
 * Base class for all modules types
 */
export class W2CModuleBase {
    constructor(name) {
        this.name = name;
        this.mangledName = mangleModuleName(name);
    }
    /**
     * Gets the C typename that was generated by wasm2c for this imported module.
     */
    get generatedContextTypeName() {
        return `w2c_${this.mangledName}`;
    }
}
/**
 * Represents an imported module in the shared context.
 *
 * This represents an imported module with imported symbols across all generated modules.
 *
 * Modules of this type have no implementation in WASM file, but the code for those modules
 * must be generated to allow defining them from JS code.
 */
export class W2CExternModule extends W2CModuleBase {
    constructor(name, symbols) {
        super(name);
        this.symbols = symbols;
    }
    /**
     * Gets the name of the field in the root context struct.
     *
     * Each generated module that imports this module contains a field that holds
     * the imported module instance.
     */
    get generatedRootContextFieldName() {
        return `import_${this.mangledName}Ctx`;
    }
}
/**
 * Represents a generated module wrapping a WebAssembly module.
 *
 * This class is created for each WebAssembly module that is processed by the `wasm2c` tool.
 */
export class W2CGeneratedModule extends W2CModuleBase {
    constructor(moduleBuffer, sourceModulePath) {
        const name = path.basename(sourceModulePath, '.wasm');
        super(name);
        this.body = new Module(moduleBuffer);
        this.sourceModulePath = sourceModulePath;
        this.checksum = computeChecksumBuffer(moduleBuffer);
        this.moduleImports = processImportedModulesInfo(this.body);
        this.imports = processImports(this, this.body);
        this.exports = processExports(this, this.body);
    }
    /**
     * Retrieves a reversed array of all imported modules.
     *
     * @return An array containing the imported modules in reverse order.
     */
    get importedModules() {
        return Object.values(this.moduleImports).toSorted((a, b) => a.name.localeCompare(b.name));
    }
    /**
     * Retrieves an array of all exported memories.
     */
    get exportedMemories() {
        return this.exports.filter((i) => i.target.kind === 'memory');
    }
    /**
     * Retrieves an array of all exported functions.
     */
    get exportedFunctions() {
        return this.exports.filter((i) => i.target.kind === 'function');
    }
    /**
     * Retrieves an array of all exported tables.
     */
    get exportedTables() {
        return this.exports.filter((i) => i.target.kind === 'table');
    }
}
function processImportedModules(modules) {
    const allImports = modules.flatMap((module) => module.codegen.imports);
    const importsGroupedByName = Object.groupBy(allImports, (el) => el.moduleInfo.name);
    const nameToInfoMap = Object.groupBy(allImports.map((el) => el.moduleInfo), (el) => el.name);
    return Object.entries(importsGroupedByName)
        .map(([moduleName, imports]) => {
        const info = nameToInfoMap[moduleName]?.[0];
        if (!info) {
            throw new Error(`Could not find info for module ${moduleName}`);
        }
        return new W2CExternModule(info, imports ?? []);
    })
        .filter(Boolean);
}
function processImportedModulesInfo(module) {
    const importedModuleNames = new Set(module.imports.values().map((i) => i.module));
    const importsInfo = importedModuleNames
        .values()
        .map((name) => [
        name,
        new W2CCodegenLocalImportedModule(name),
    ]);
    return Object.fromEntries(importsInfo);
}
function processImports(module, body) {
    return body.imports.map((el) => {
        const importInfo = module.moduleImports[el.module];
        if (!importInfo) {
            throw new Error(`Assert error: could not get import info for ${el.module}`);
        }
        const generatedFunctionName = mangleFunction(el.name, importInfo.mangledName);
        switch (el.target.kind) {
            case 'function':
                return {
                    ...el,
                    ...buildGeneratedFunctionInfo(el.target),
                    mangledName: mangleName(el.name),
                    moduleInfo: importInfo,
                    generatedFunctionName,
                };
            default:
                return {
                    ...el,
                    mangledName: mangleName(el.name),
                    moduleInfo: importInfo,
                    generatedFunctionName,
                };
        }
    });
}
function processExports(context, module) {
    return module.exports.map((el) => {
        const mangledName = mangleName(el.name);
        const generatedFunctionName = mangleFunction(el.name, context.mangledName);
        switch (el.target.kind) {
            case 'function':
                return {
                    ...el,
                    ...buildGeneratedFunctionInfo(el.target),
                    generatedFunctionName,
                    mangledName,
                };
            default:
                return {
                    ...el,
                    mangledName,
                    generatedFunctionName,
                };
        }
    });
}
function mangleFunction(name, mangledModule) {
    return `w2c_${mangledModule}_${mangleName(name)}`;
}
function buildGeneratedFunctionInfo(func) {
    const returnType = matchW2CRType(func.resultTypes[0]);
    return {
        parameterTypeNames: func.parametersTypes.map(matchW2CRType),
        returnTypeName: returnType,
    };
}
// TODO: workaround a bug(?) that all types are returned as either none, u32 or f64
function matchW2CRType(t) {
    if (!t) {
        return 'void';
    }
    // TODO: figure out why wasm2c returns u32 for number types sometimes
    if (t.startsWith('i')) {
        return t.replace(/^i/, 'u');
    }
    return t;
}
