import { types } from './builder.types.js';
import { SourceWriter } from './source-writer.js';
export class SourceBuilderError extends Error {
    constructor(message) {
        super(message);
        this.name = 'SourceBuilderError';
    }
}
export class SourceBlockBuilder {
    constructor(isHeader, writer) {
        this.isHeader = isHeader;
        this.writer = writer ?? new SourceWriter();
    }
    //
    // ------------------------------------------------------------------------------------------ UTILITIES
    //
    /**
     * Returns the generated source code as a string.
     */
    toString() {
        return this.writer.toString();
    }
    /**
     * Adds vertical spacing to the source code.
     *
     * @param lines
     */
    spacing(lines = 1) {
        for (let i = 0; i < lines; ++i) {
            this.writer.writeLine('');
        }
        return this;
    }
    /**
     * Helper function to generate a type.
     *
     * @param builder Function that builds a type
     * @return C++ Type as string
     */
    type(builder) {
        return builder(types).toString();
    }
    /**
     * Inserts generated code for each element in the array using custom generator.
     *
     * @param elements
     * @param generator
     */
    writeManyLines(elements, generator) {
        elements.forEach((element) => {
            this.writer.writeLine(generator(element, this));
        });
        return this;
    }
    //
    // ------------------------------------------------------------------------------------------ C++ STATEMENTS
    //
    /**
     * Adds a using namespace directive.
     * @param name Namespace to use
     */
    usingNamespace(name) {
        this.writer.writeLine(`using namespace ${name};`);
        return this;
    }
    using(name, value) {
        this.writer.writeLine(`using ${name} = ${value};`);
        return this;
    }
    return_(target) {
        this.writer.writeLine(`return ${target};`);
        return this;
    }
    //
    // ------------------------------------------------------------------------------------------ C++ DECLARATIONS
    //
    namespace(name, action) {
        this.writer.writeLine(`namespace ${name} {`);
        this.spacing(1);
        action(this);
        this.spacing(1);
        this.writer.writeLine('}');
        return this;
    }
    defineFunction(builder) {
        this.writer.writeLine('');
        builder.build(this.writer);
        this.spacing();
        return this;
    }
    defineVariable(builder) {
        this.writer.writeLine('');
        builder.build(this.writer);
        this.spacing();
        return this;
    }
    // UNSAFE
    insertRaw(source) {
        for (const line of source.split('\n')) {
            this.writer.writeLine(line);
        }
        return this;
    }
}
