import { SourceWriter } from "./source-writer.js";
export class SourceBuilderError extends Error {
    constructor(message) {
        super(message);
        this.name = 'SourceBuilderError';
    }
}
export class SourceBuilder {
    constructor(isHeader, writer) {
        this.isHeader = isHeader;
        this.writer = writer ?? new SourceWriter();
    }
    //
    // ------------------------------------------------------------------------------------------ UTILITIES
    //
    /**
     * Returns the generated source code as a string.
     */
    toString() {
        return this.writer.toString();
    }
    /**
     * Adds vertical spacing to the source code.
     *
     * @param lines
     */
    spacing(lines = 1) {
        for (let i = 0; i < lines; ++i) {
            this.writer.writeLine('');
        }
        return this;
    }
    /**
     * Helper function to generate a type.
     *
     * @param builder Function that builds a type
     * @return C++ Type as string
     */
    type(builder) {
        return builder(types);
    }
    /**
     * Inserts generated code for each element in the array using custom generator.
     *
     * @param elements
     * @param generator
     */
    insertGenerated(elements, generator) {
        elements.forEach((element) => {
            this.writer.writeLine(generator(element, this));
        });
        return this;
    }
    //
    // ------------------------------------------------------------------------------------------ C++ STATEMENTS
    //
    /**
     * Adds a using namespace directive.
     * @param name Namespace to use
     */
    usingNamespace(name) {
        this.writer.writeLine(`using namespace ${name};`);
        return this;
    }
    using(name, value) {
        this.writer.writeLine(`using ${name} = ${value};`);
        return this;
    }
    //
    // ------------------------------------------------------------------------------------------ C++ DECLARATIONS
    //
    namespace(name, block) {
        this.writer.writeLine(`namespace ${name} {`);
        this.writer.withIndent(() => block(this));
        this.writer.writeLine('}');
        return this;
    }
    function(name, block) {
        this.writer.writeLine(`namespace ${name} {`);
        this.writer.pushIndent();
        block(this);
        this.writer.popIndent();
        this.writer.writeLine('}');
        return this;
    }
}
export class SourceFileBuilder extends SourceBuilder {
    constructor(writer) {
        super(false, writer);
    }
    //
    // ------------------------------------------------------------------------------------------ C++ STATEMENTS
    //
    writeIncludeGuard() {
        if (!this.isHeader) {
            throw new SourceBuilderError('Cannot add include guard to source file');
        }
        this.writer.writeLine(`#pragma once`);
        return this;
    }
    /**
     * Adds an include preprocesor directive.
     *
     * @param path Path to include
     * @param local Whenever to use local include or a system include
     */
    include(path, local) {
        if (local) {
            this.writer.writeLine(`#include "${path}"`);
        }
        else {
            this.writer.writeLine(`#include <${path}>`);
        }
        return this;
    }
    /**
     * Adds a local include preprocesor directive.
     *
     * @param path Path to include
     */
    includeLocal(path) {
        return this.include(path, true);
    }
    /**
     * Adds a system include preprocesor directive.
     *
     * @param path Path to include
     */
    includeSystem(path) {
        return this.include(path, false);
    }
}
