/**
 * SourceWriter is a helper class to build source code strings with proper indentation.
 */
export class SourceWriter {
    constructor(indentLevel = 0, indentSize = 2) {
        this.source = '';
        this.indentLevel = indentLevel;
        this.indentSize = indentSize;
    }
    /**
     * Write a string to the source code.
     *
     * @param str String to write
     */
    write(str) {
        if (str.includes('\n') && !str.endsWith('\n')) {
            for (const line of str.split('\n')) {
                this.source += this.indentString(line + '\n');
            }
        }
        else {
            this.source += this.indentString(str);
        }
        return this;
    }
    /**
     * Write a line to the source code.
     *
     * @param line Line to write
     */
    writeLine(line) {
        return this.write(`${line}\n`);
    }
    /**
     * Write multiple items to the source code.
     *
     * @param elements Elements to write
     * @param builder Builder function to generate the line for each element
     * @param joiner How to join the generated elements
     */
    writeMany(elements, builder, joiner) {
        elements.forEach((element, index) => {
            const res = builder(element, this);
            if (typeof res === 'string') {
                this.write(res);
            }
            if (index < elements.length - 1) {
                this.write(joiner);
            }
        });
        return this;
    }
    /**
     * Write a list of elements separated by a comma.
     *
     * @param elements Elements to write
     * @param builder Builder function to generate the source for each element
     */
    writeCommaList(elements, builder) {
        return this.writeMany(elements, builder, ', ');
    }
    /**
     * Increase the indentation level.
     */
    pushIndent() {
        this.indentLevel += this.indentSize;
        return this;
    }
    /**
     * Decrease the indentation level.
     */
    popIndent() {
        this.indentLevel -= this.indentSize;
        return this;
    }
    /**
     * Write a block of code with increased indentation.
     *
     * @param cb Callback to use to write the indented block
     */
    withIndent(cb) {
        this.pushIndent();
        cb();
        this.popIndent();
        return this;
    }
    /**
     * Returns the generated source code as a string.
     */
    toString() {
        return this.source;
    }
    indentString(str) {
        if (this.isFreshLine) {
            return ' '.repeat(this.indentLevel * this.indentSize) + str;
        }
        return str;
    }
    get isFreshLine() {
        return this.source.endsWith('\n');
    }
}
