import { Module, type ModuleMemory, ModuleTable } from '@callstack/wasm-parser';
import type { GeneratedExport, GeneratedFunctionExport, GeneratedImport } from '../types.js';
type ImportMap = Record<string, W2CCodegenLocalImportedModule>;
/**
 * The `W2CModuleCodegenContext` class is responsible for maintaining the state and metadata
 * related to the code generation process of a WebAssembly (WASM) module using the `wasm2c` tool.
 * This class stores information about module imports and exports, and provides utilities
 * for handling the generated module code.
 */
export declare class W2CModuleCodegenContext {
    /**
     * Collection of imported modules and their mangled names.
     */
    readonly moduleImports: ImportMap;
    /**
     * Mangled module name, safe to use in source code as a symbol name.
     */
    readonly mangledName: string;
    /**
     * Collection of generated module imports made by `wasm2c` tool.
     */
    readonly imports: GeneratedImport[];
    /**
     * Collection of generated module exports made by `wasm2c` tool.
     */
    readonly exports: GeneratedExport[];
    constructor(name: string, module: Module);
    /**
     * Gets a C typename that was generated by wasm2c for this module.
     */
    get generatedContextTypeName(): string;
    /**
     * Retrieves a reversed array of all imported modules.
     *
     * @return An array containing the imported modules in reverse order.
     */
    get importedModules(): W2CCodegenLocalImportedModule[];
    /**
     * Retrieves an array of all exported memories.
     */
    get exportedMemories(): GeneratedExport<ModuleMemory>[];
    /**
     * Retrieves an array of all exported functions.
     */
    get exportedFunctions(): GeneratedFunctionExport[];
    /**
     * Retrieves an array of all exported tables.
     */
    get exportedTables(): GeneratedExport<ModuleTable>[];
}
/**
 * Represents a WebAssembly to C (W2C) imported module with details about its name and mangled name.
 *
 * This class provides functionalities to handle and retrieve the context type
 * and root context field names for the imported module based on its mangled name.
 *
 * @remarks The mangled name is derived from the original module name through a specific mangle transformation.
 */
export declare class W2CCodegenLocalImportedModule {
    /**
     * Original name of the imported module.
     */
    readonly name: string;
    /**
     * Mangled name of the imported module.
     */
    readonly mangledName: string;
    constructor(name: string);
    /**
     * Gets the C typename that was generated by wasm2c for this imported module.
     */
    get generatedContextTypeName(): string;
    /**
     * Gets the name of field of root context struct, that was generated by wasm2c for this imported module.
     */
    get generatedRootContextFieldName(): string;
}
export {};
//# sourceMappingURL=codegen-context.d.ts.map