/**
 * Represents a map of written files and their last modified times.
 */
export type WrittenFilesMap = Record<string, number>;
/**
 * Error thrown when a file would be overwritten by the output generator.
 */
export declare class FileOverwriteError extends Error {
    path: string;
    constructor(path: string);
}
/**
 * Error thrown when a file would be overwritten by the output generator.
 */
export declare class FileExternallyChangedError extends Error {
    path: string;
    constructor(path: string);
}
/**
 * Represents configuration options for the output generator.
 *
 * This interface is used to configure the behavior of the output generation process,
 * such as specifying directories for assets and output, and optionally enforcing
 * regeneration of outputs.
 */
export interface OutputGeneratorOptions {
    /**
     * Path to the directory containing the assets to be used during the generation process.
     */
    assetsDirectory: string;
    /**
     * Path to the directory where the generated output should be stored.
     */
    outputDirectory: string;
    /**
     * An optional flag that, if set to true, forces the generation process to overwrite existing outputs.
     */
    forceGenerate?: boolean;
}
/**
 * Helper class that generates output files and copies assets.
 */
export declare class OutputGenerator {
    /**
     * Configuration options for the output generator.
     */
    readonly options: OutputGeneratorOptions;
    /**
     * Set of files that have been written by the output generator.
     */
    readonly writtenFiles: WrittenFilesMap;
    /**
     * A map of previously written files, used to determine if the output files are outdated.
     */
    private readonly previousWrittenFiles?;
    /**
     * Local path within output directory that is currently bound to this output generator.
     *
     * For root generator this is empty, for any generator created by `forPath` method this
     * is the path that was passed to it.
     */
    private boundNestedPath;
    /**
     * Root output generator instance.
     *
     * For root generator this is the same instance, for any generator created by `forPath` method
     * this is the root generator instance.
     */
    private rootGenerator;
    constructor(options: OutputGeneratorOptions, previousWrittenFiles?: WrittenFilesMap);
    /**
     * Generates an OutputGenerator with an updated output directory by appending the given nested directory path
     * to the base output directory.
     *
     * @param nestedDirPath The relative path to the nested directory to append to the output directory.
     * @return A new OutputGenerator instance with the updated output directory path.
     */
    forPath(nestedDirPath: string): OutputGenerator;
    markAsWritten(filePath: string): Promise<void>;
    /**
     * Builds path to specified file, prefixing it with output directory, if any.
     *
     * @param names Path components
     */
    outputPathTo(...names: string[]): string;
    /**
     * Writes the provided text to the specified file path, creating any necessary directories.
     *
     * @param to - The relative or absolute path where the text should be written.
     * @param text - The content to write to the specified file.
     * @return A promise that resolves once the text has been successfully written.
     */
    writeTo(to: string, text: string): Promise<void>;
    /**
     * Writes all content from the given map to respective destinations.
     *
     * @param contentMap A record where keys are destination paths and values are the content to be written.
     * @return A promise that resolves when all write operations have completed.
     */
    writeAllTo(contentMap: Record<string, string>): Promise<void>;
    /**
     * Copies an asset from the specified asset path to a target destination.
     *
     * @param assetPath The relative path of the asset to be copied from the assets directory.
     * @param to The target path where the asset should be copied. Defaults to the original asset path.
     * @return A promise that resolves when the asset copy operation is complete.
     */
    copyAsset(assetPath: string, to?: string): Promise<void>;
    /**
     * Copies assets from the specified source locations to the corresponding target locations.
     *
     * @param assetMap A record where the key represents the source path of the asset,
     * and the value represents the target path where the asset should be copied.
     * @return A promise that resolves when all asset copy operations have been completed.
     */
    copyAssets(assetMap: Record<string, string>): Promise<void>;
    /**
     * Helper method that generates specified files only when the source files
     * are newer than output files, which suggest that the output files are outdated.
     *
     * The comparison is done basing on `modified_time` attribute of every file.
     *
     * @param sources List of input files
     * @param outputs List of output files
     * @param cb Callback that generates the output files
     * @protected
     */
    generating<R>(sources: string[], outputs: string[], cb: () => R): Promise<R | void>;
    private assertSafeToWrite;
}
//# sourceMappingURL=output-generator.d.ts.map